﻿//http://blog.codeville.net/2008/10/15/partial-output-caching-in-aspnet-mvc/
namespace AtomSite.WebCore
{
  using System;
  using System.IO;
  using System.Reflection;
  using System.Text;
  using System.Web;
  using System.Web.Caching;
  using System.Web.Mvc;
  using System.Web.UI;
  using AtomSite.Domain;
  using StructureMap;

  public class ActionOutputCacheAttribute : ActionFilterAttribute
  {
    // This hack is optional; I'll explain it later in the blog post
    private static MethodInfo _switchWriterMethod = typeof(HttpResponse).GetMethod("SwitchWriter", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);


    public ActionOutputCacheAttribute(int cacheDuration)
      : this(cacheDuration, false)
    { }

    public ActionOutputCacheAttribute(int cacheDuration, bool anonOnly)
    {
      _cacheDuration = cacheDuration;
      _anonOnly = anonOnly;
    }

    private int _cacheDuration;
    private bool _anonOnly;
    private TextWriter _originalWriter;
    private string _cacheKey;

    public override void OnActionExecuting(ActionExecutingContext filterContext)
    {
      _cacheKey = ComputeCacheKey(filterContext);
      string cachedOutput = (string)filterContext.HttpContext.Cache[_cacheKey];
      if (cachedOutput != null)
        filterContext.Result = new ContentResult { Content = cachedOutput };
      else
        _originalWriter = (TextWriter)_switchWriterMethod.Invoke(HttpContext.Current.Response, new object[] { new HtmlTextWriter(new StringWriter()) });
    }

    public override void OnResultExecuted(ResultExecutedContext filterContext)
    {
      if (_originalWriter != null) // Must complete the caching
      {
        HtmlTextWriter cacheWriter = (HtmlTextWriter)_switchWriterMethod.Invoke(HttpContext.Current.Response, new object[] { _originalWriter });
        string textWritten = ((StringWriter)cacheWriter.InnerWriter).ToString();
        filterContext.HttpContext.Response.Write(textWritten);

        filterContext.HttpContext.Cache.Add(_cacheKey, textWritten, null, DateTime.Now.AddSeconds(_cacheDuration), Cache.NoSlidingExpiration, System.Web.Caching.CacheItemPriority.Normal, null);
      }
    }

    private string ComputeCacheKey(ActionExecutingContext filterContext)
    {
      //if anon only return guid when not anon
      if (_anonOnly && !filterContext.HttpContext.User.Identity.IsAuthenticated) return Guid.NewGuid().ToString();

      var keyBuilder = new StringBuilder();
      keyBuilder.Append(filterContext.ActionDescriptor.ControllerDescriptor.ControllerName);
      keyBuilder.Append(filterContext.ActionDescriptor.ActionName);

      //factor authorization roles into cache key
      BaseController c = filterContext.Controller as BaseController;
      if (c != null)
      {
        IAuthorizeService auth = ((IContainer)filterContext.HttpContext.Application["container"]).GetInstance<IAuthorizeService>();
        AuthRoles roles = auth.GetRoles((User)filterContext.HttpContext.User.Identity, c.Scope);
        keyBuilder.AppendFormat("r" + roles);
      }
      //do not factor route into caching since may be cached across multiple pages
      //foreach (var pair in filterContext.RouteData.Values)
      //  keyBuilder.AppendFormat("rd{0}_{1}_", pair.Key.GetHashCode(), pair.Value.GetHashCode());


      foreach (var pair in filterContext.ActionParameters)
      {
        if (pair.Value != null)
          keyBuilder.AppendFormat("ap{0}_{1}_", pair.Key.GetHashCode(), pair.Value.GetHashCode());
      }
      return keyBuilder.ToString();
    }
  }
}
