﻿/* Copyright 2008-10 Jarrett Vance
* see http://www.codeplex.com/blogsvc/license */
namespace AtomSite.WebCore
{
  using System.Collections.Generic;
  using System.Linq;
  using AtomSite.Domain;

  public interface IPage
  {
    string Name { get; }
    string Description { get; }
    string Parent { get; }
    IEnumerable<Asset> Assets { get; }
    SupportedScopes SupportedScopes { get; }
    void UpdatePageModel(PageModel pageModel);
  }

  public class Page : IPage
  {
    public Page(string name)
    {
      Name = name;
      Assets = new Asset[] { };
    }

    public string Name { get; protected set; }
    public string Description { get; set; }
    public string Parent { get; set; }
    public IEnumerable<Asset> Assets { get; set; }
    public SupportedScopes SupportedScopes { get; set; }
    public void UpdatePageModel(PageModel pageModel)
    {
      foreach (Asset asset in Assets) pageModel.Assets.Add(asset);
    }

    protected virtual Asset AddAsset(string assetName)
    {
      var asset = new Asset(assetName) { AssetScope = AssetScope.Widget };
      Assets = Assets.Concat(new[] { asset });
      return asset;
    }
    protected virtual Asset AddAsset(string assetName, string assetGroup)
    {
      var asset = new Asset(assetName, assetGroup, AssetScope.Widget);
      Assets = Assets.Concat(new[] { asset });
      return asset;
    }
  }

}
