﻿/* Copyright 2008-10 Jarrett Vance
* see http://www.codeplex.com/blogsvc/license */
namespace AtomSite.WebCore
{
  using System;
  using System.Linq;
  using System.Web;
  using System.Web.Mvc;
  using AtomSite.Domain;
  public partial class AdminController : BaseController
  {
    [ScopeAuthorize]
    public ViewResult Entries(string workspace, string collection, string filter, int? page, int? pageSize)
    {
      int pageIndex = page - 1 ?? 0;
      int pSize = pageSize ?? AppService.GetCollection().PageSize;

      var entries = AtomPubService.GetEntries(
        filter == "pending" ? SelectionCriteria.EntriesPending(workspace, collection) :
        filter == "published" ? SelectionCriteria.EntriesPublished(workspace, collection) :
        filter == "draft" ? SelectionCriteria.EntriesDraft(workspace, collection) :
        filter == "unapproved" ? SelectionCriteria.EntriesUnapproved(workspace, collection) :
        SelectionCriteria.EntriesAll(workspace, collection), pageIndex, pSize);

      var m = new AdminEntriesModel() { Entries = entries };
      m.AllCount = AtomEntryRepository.GetEntriesCount(SelectionCriteria.EntriesAll(workspace, collection));
      m.PublishedCount = AtomEntryRepository.GetEntriesCount(SelectionCriteria.EntriesPublished(workspace, collection));
      m.UnapprovedCount = AtomEntryRepository.GetEntriesCount(SelectionCriteria.EntriesUnapproved(workspace, collection));
      m.DraftCount = AtomEntryRepository.GetEntriesCount(SelectionCriteria.EntriesDraft(workspace, collection));
      m.Filter = filter;
      return View("AdminEntries", "Admin", m);
    }

    [ScopeAuthorize]
    [System.Web.Mvc.AcceptVerbs(HttpVerbs.Get)]
    public ActionResult EditEntry(string workspace, string collection, string id)
    {
      //TODO: move logic to service
      AtomEntry entry = null;
      if (TempData["entry"] != null) entry = TempData["entry"] as AtomEntry;
      if (entry == null)
      {
        if (id != null) entry = AtomPubService.GetEntry(id);
        else if (Collection != null && Collection.OnlyAcceptsMedia)
        {
          return RedirectToAction("EditMedia", new
          {
            workspace = Collection.Id.Workspace,
            collection = Collection.Id.Collection
          });
        }
        else if (Collection == null)
        {
          //find entry collection in approved scopes
          var scopes = AuthorizeService.GetScopes(User.Identity as User);
          var coll = AppService.GetCollections(scopes).Where(c => c.AcceptsEntries).FirstOrDefault()
            ?? AppService.GetCollection();

          return RedirectToAction("EditEntry", new
          {
            workspace = coll.Id.Workspace,
            collection = coll.Id.Collection
          });
        }
        else
        {
          entry = new AtomEntry()
          {
            Title = new AtomTitle() { Text = "Enter Title" },
            Content = new AtomContent() { Text = string.Empty }
          };
          //TODO: workspace/collection?
        }
      }
      if (entry.Media) throw new Exception("Can't edit media entry");
      if (Collection.OnlyAcceptsMedia) throw new Exception("Selected collection only accepts media.");
      if (!Collection.AcceptsEntries) throw new Exception("Selected collection does not accept new entries.");
      var m = new AdminEntryModel() { Entry = entry };
      if (TempData["error"] != null) m.Errors.Add((string)TempData["error"]);
      if (TempData["created"] != null) m.Notifications.Add("Success", "The new entry '" + entry.Id.EntryPath + "' was successfully created.");
      if (TempData["updated"] != null) m.Notifications.Add("Success", "The existing entry '" + entry.Id.EntryPath + "' was successfully updated.");
      return View("AdminEditEntry", "Admin", m);
    }


    [ScopeAuthorize]
    [System.Web.Mvc.AcceptVerbs(HttpVerbs.Get)]
    public ActionResult EditMedia(string workspace, string collection, string id)
    {
      AtomEntry entry = null;
      if (TempData["entry"] != null) entry = TempData["entry"] as AtomEntry;
      if (entry == null)
      {
        if (id != null) entry = AtomPubService.GetEntry(id);
        else if (Collection != null && Collection.OnlyAcceptsEntries)
        {
          return RedirectToAction("EditEntry", new
          {
            workspace = Collection.Id.Workspace,
            collection = Collection.Id.Collection
          });
        }
        else if (Collection == null || !Collection.AcceptsMedia)
        {
          //find media collection in approved scopes
          var scopes = AuthorizeService.GetScopes(User.Identity as User);
          var coll = AppService.GetCollections(scopes).Where(c => c.AcceptsMedia).FirstOrDefault()
            ?? AppService.GetCollection();

          return RedirectToAction("EditMedia", new
          {
            workspace = coll.Id.Workspace,
            collection = coll.Id.Collection
          });
        }
        else
        {
          entry = new AtomEntry()
          {
            Title = new AtomTitle() { Text = "Enter Title" },
            Content = new AtomContent() { Text = string.Empty },
            Media = true
          };
          //TODO: workspace/collection?
        }
      }
      if (!entry.Media) throw new Exception("Can't edit non-media entry");
      if (Collection.OnlyAcceptsEntries) throw new Exception("Selected collection does not accept media.");
      var m = new AdminMediaModel() { Entry = entry };
      if (TempData["error"] != null) m.Errors.Add((string)TempData["error"]);
      if (TempData["created"] != null) m.Notifications.Add("Success", "The new media '" + entry.Id.EntryPath + "' was successfully created.");
      if (TempData["updated"] != null) m.Notifications.Add("Success", "The existing media '" + entry.Id.EntryPath + "' was successfully updated.");
      return View("AdminEditMedia", "Admin", m);
    }

    [ScopeAuthorize]
    [System.Web.Mvc.AcceptVerbs(HttpVerbs.Post)]
    [System.Web.Mvc.ValidateInput(false)]
    public ActionResult EditMedia(string workspace, string collection, string id, string title, string slug, string summary, bool? draft, bool? approved, string publishedDate, string publishedTime, string submit, string[] categories, bool? allowAnnotations)
    {
      
      //TODO: use ModelBinder?
      AtomEntry entry = null;
      if (title == "Enter Title") title = null;
      if (string.IsNullOrEmpty(slug)) slug = title;

      try
      {
        if (Request.Files.Count == 1)
        {
          HttpPostedFileBase hpf = Request.Files[0] as HttpPostedFileBase;
          if (hpf.ContentLength == 0) throw new Exception("Empty file uploaded");
          if (string.IsNullOrEmpty(slug)) slug = title = hpf.FileName;

          if (!string.IsNullOrEmpty(id))
          {
            entry = AtomPubService.UpdateMedia(id, hpf.InputStream, hpf.ContentType);
            TempData["updated"] = true;
          }
          else
          {
            entry = AtomPubService.CreateMedia(AppService.GetCollection(workspace, collection).Id,
            hpf.InputStream, slug, hpf.ContentType);
            TempData["created"] = true;
          }
        }
        if (!string.IsNullOrEmpty(id)) entry = AtomPubService.GetEntry(id);

        if (entry == null) throw new Exception("File not uploaded");

        //TODO: DRY violation with EditEntry
        if (string.IsNullOrEmpty(title)) throw new Exception("Title is required");
        entry.Title = new AtomTitle() { Text = title };

        if (!string.IsNullOrEmpty(summary)) entry.Summary = new AtomSummary() { Text = summary };
        else entry.Summary = null;

        if (entry.Control == null) entry.Control = new AppControl();

        entry.Control.Draft = draft;
        if (submit != "Publish") entry.Control.Draft = true;

        entry.Control.Approved = approved;

        if (publishedDate != null && publishedTime != null)
        {
          DateTimeOffset dto;
          if (DateTimeOffset.TryParse(publishedDate + " " + publishedTime, out dto))
          {
            TimeZoneInfo tz = TimeZoneInfo.FindSystemTimeZoneById(Properties.Settings.Default.TimeZoneInfoId);
            dto = new DateTimeOffset(dto.Year, dto.Month, dto.Day, dto.Hour, dto.Minute, dto.Second, tz.BaseUtcOffset);// TimeZoneInfo.ConvertTime(dto, tz);
            entry.Published = dto;
          }
          else throw new Exception("The publish date or time was invalid.  Please enter a valid date and time.");
        }

        if (allowAnnotations.HasValue) entry.Control.AllowAnnotate = allowAnnotations;
        else entry.Control.AllowAnnotate = null;

        if (categories != null)
        {
          var cats = categories.Select(c => new AtomCategory() { Term = c });
          entry.Categories = cats;
        }

        AtomPubService.UpdateEntry(entry.Id, entry, slug);

        if (submit == "Preview")
          return Redirect(entry.LocationWeb.ToString());

        return RedirectToAction("EditMedia", new { id = entry.Id.ToString() });
      }
      catch (Exception ex)
      {
        LogService.Error(ex);
        TempData["entry"] = entry;
        TempData["error"] = ex.Message;
        if (id != null)
          return RedirectToAction("EditMedia", new { id = id });
      }
      return RedirectToAction("EditMedia", new { workspace = workspace, collection = collection });
    }

    [ScopeAuthorize]
    [System.Web.Mvc.AcceptVerbs(HttpVerbs.Post)]
    [System.Web.Mvc.ValidateInput(false)]
    public ActionResult EditEntry(string workspace, string collection, string id, string title, string slug, string content, string summary, bool? draft, bool? approved, string publishedDate, string publishedTime, string submit, string[] categories, bool? allowAnnotations)
    {
      //TODO: use ModelBinder?
      AtomEntry entry = null;
      try
      {
        //TODO: verify contents
        content = "<div xmlns=\"http://www.w3.org/1999/xhtml\">" + content.Replace("&nbsp;", "&#160;") + "</div>";

        if (!string.IsNullOrEmpty(id)) entry = AtomPubService.GetEntry(id);
        else entry = new AtomEntry();

        if (string.IsNullOrEmpty(title) || title == "Enter Title") title = slug;
        if (string.IsNullOrEmpty(title)) throw new Exception("Title is required");
        entry.Title = new AtomTitle() { Text = title };
        entry.Content = new AtomContent() { Type = "xhtml", Text = content };

        if (!string.IsNullOrEmpty(summary)) entry.Summary = new AtomSummary() { Text = summary };
        else entry.Summary = null;

        if (entry.Control == null) entry.Control = new AppControl();

        entry.Control.Draft = draft;
        if (submit != "Publish") entry.Control.Draft = true;

        entry.Control.Approved = approved;

        if (publishedDate != null && publishedTime != null)
        {
          DateTimeOffset dto;
          if (DateTimeOffset.TryParse(publishedDate + " " + publishedTime, out dto))
          {
            TimeZoneInfo tz = TimeZoneInfo.FindSystemTimeZoneById(Properties.Settings.Default.TimeZoneInfoId);
            dto = new DateTimeOffset(dto.Year, dto.Month, dto.Day, dto.Hour, dto.Minute, dto.Second, tz.BaseUtcOffset);// TimeZoneInfo.ConvertTime(dto, tz);
            entry.Published = dto;
          }
          else throw new Exception("The publish date or time was invalid.  Please enter a valid date and time.");
        }

        if (allowAnnotations.HasValue) entry.Control.AllowAnnotate = allowAnnotations;
        else entry.Control.AllowAnnotate = null;

        if (categories != null)
        {
          var cats = categories.Select(c => new AtomCategory() { Term = c });
          entry.Categories = cats;
        }

        if (string.IsNullOrEmpty(id))
        {
          AtomPubService.CreateEntry(Collection.Id, entry, slug);
          TempData["created"] = true;
        }
        else
        {
          AtomPubService.UpdateEntry(id, entry, slug);
          TempData["updated"] = true;
        }

        if (submit == "Preview")
          return Redirect(entry.LocationWeb.ToString());

        return RedirectToAction("EditEntry", new { id = entry.Id.ToString() });
      }
      catch (Exception ex)
      {
        LogService.Error(ex);
        TempData["error"] = ex.Message;
        TempData["entry"] = entry;
        if (id != null)
          return RedirectToAction("EditEntry", new { id = id });
      }
      return RedirectToAction("EditEntry");
    }

  }
}
