﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using AtomSite.Domain;
using System.Web.Mvc;
using AtomSite.Repository;

namespace AtomSite.WebCore
{
  public partial class AdminController : BaseController
  {

    [ScopeAuthorize]
    public ViewResult Theme(string workspace, string collection)
    {
      var m = new AdminThemeModel();
      m.CurrentThemeName = ThemeService.GetThemeName(Scope);
      m.InheritedThemeName = ThemeService.GetInheritedThemeName(Scope);
      m.Inherited = ThemeService.GetThemeName(Scope) == null;
      m.Theme = ThemeService.GetTheme(m.InheritedThemeName);
      m.ThemeLocation = ThemeService.GetThemePath(m.InheritedThemeName);
      m.InstalledThemes = ThemeService.GetInstalledThemes().Select(t => ThemeService.GetTheme(t));
      if (TempData["error"] != null) m.Errors.Add((string)TempData["error"]);
      if (TempData["applied"] != null) m.Notifications.Add("Success", "The theme was successfully updated.");
      //temporary notification
      else if (m.Errors.Count == 0) m.Notifications.Add("So you've mastered CSS,", "then help us improve the theme system by creating a theme.");
      return View("AdminTheme", "Admin", m);
    }

    [ScopeAuthorize]
    public PartialViewResult ThemeInfo(string workspace, string collection, string theme)
    {
      var m = new AdminThemeModel();
      m.CurrentThemeName = ThemeService.GetThemeName(Scope);
      m.InheritedThemeName = ThemeService.GetInheritedThemeName(Scope);
      if (theme == "(inherit)")
      {
        theme = ThemeService.GetInheritedThemeName(Scope.ToAbove());
        m.Inherited = true;
      }
      m.Theme = ThemeService.GetTheme(theme);
      m.ThemeLocation = ThemeService.GetThemePath(theme);
      m.InstalledThemes = ThemeService.GetInstalledThemes().Select(t => ThemeService.GetTheme(t));
      return PartialView("AdminThemeInfo", m);
    }

    [ScopeAuthorize]
    public ActionResult ChangeTheme(string workspace, string collection, string theme, string submit)
    {
      try
      {
        if (theme == "(inherit)") theme = null;
        if (submit == "Apply Theme")
        {
          AdminService.ApplyTheme(Scope, theme);
          TempData["applied"] = true;
        }
        else throw new NotImplementedException();
        return RedirectToAction("Theme", new { workspace = workspace, collection = collection });
      }
      catch (Exception ex)
      {
        LogService.Error(ex);
        TempData["error"] = ex.Message;
        return RedirectToAction("Theme", new { workspace = workspace, collection = collection });
      }
    }
  }
}
