﻿/* Copyright 2008-10 Jarrett Vance
* see http://www.codeplex.com/blogsvc/license */
namespace AtomSite.WebCore
{
  using System.Collections.Generic;
  using System.Web.Mvc;
  using System.Web.Routing;
  using System.Linq;
  using AtomSite.Repository;
  using AtomSite.Domain;

  public class AssetPlugin : BasePlugin
  {
    public AssetPlugin(ILogService logger)
      : base(logger)
    {
      DefaultMerit = (int)MeritLevel.Max;
    }

    public override AtomSite.Domain.PluginState Upgrade(StructureMap.IContainer container, string appPath, System.Version previous, System.Version current)
    {
      //fix non named pages to specify group
      if (previous == ServerApp.Version12)
      {
        LogService.Info("Updating non named page includes in configuration to the default group");
        var svcRepo = container.GetInstance<IAppServiceRepository>();
        var appSvc = svcRepo.GetService();
        bool changes = false;
        foreach (var page in appSvc.Pages.Where(p => string.IsNullOrEmpty(p.Name) && string.IsNullOrEmpty(p.Group)))
        {
          page.Group = Asset.DefaultGroup;
          changes = true;
        }
        if (changes)
        {
          LogService.Info("Saving changes to service configuration");
          svcRepo.UpdateService(appSvc);
        }
      }
      return base.Upgrade(container, appPath, previous, current);
    }

    public override void Register(StructureMap.IContainer container, List<SiteRoute> routes, System.Web.Mvc.ViewEngineCollection viewEngines, System.Web.Mvc.ModelBinderDictionary modelBinders, ICollection<AtomSite.Domain.Asset> assets)
    {
      RegisterController<AssetController>(container);

      this.AddRoute(routes, new SiteRoute()
      {
        Merit = DefaultMerit,
        Name = "AssetGroupJs",
        Route = new System.Web.Routing.Route("js/combined/{group}.js",
          new RouteValueDictionary(new { controller = "Asset", action = "GroupJs" }), new MvcRouteHandler())
      });
      //routes.Insert(1, new SiteRoute()
      //{
      //  Merit = (int)MeritLevel.Max,
      //  Name = "AssetCssIgnore",
      //  Route = new System.Web.Routing.Route("css/*", new StopRoutingHandler())
      //});
      this.AddRoute(routes, new SiteRoute()
      {
        Merit = DefaultMerit,
        Name = "AssetGroupCss",
        Route = new System.Web.Routing.Route("css/combined/{group}.css",
          new RouteValueDictionary(new { controller = "Asset", action = "GroupCss" }), new MvcRouteHandler())
      });
      //routes.Insert(3, new SiteRoute()
      //{
      //  Merit = (int)MeritLevel.Max,
      //  Name = "AssetJsIgnore",
      //  Route = new System.Web.Routing.Route("js/*", new StopRoutingHandler())
      //});
    }
  }
}
